#pragma once

#include "Matrix.h"

namespace GraceAI
{
	class Layer
	{
		// Activations after activation function
		Matrix* activations;

		// Activations prior to activation function
		Matrix* sums;

		// Weights to previous layer
		Matrix* weights;

		// Bias
		Matrix* bias;

		// Change in activations
		Matrix* deltaActivations;

		// Change in sums
		Matrix* deltaSums;

		// Change in weights
		Matrix* deltaWeights;

		// Change in bias
		Matrix* deltaBias;

		// Neuron count for this layer
		int neuronCount;

		// Records for previous and next layers in the network
		Layer* previousLayer;
		Layer* nextLayer;
	public:

		// Public constructor creates a new layer with neuronCount neurons
		Layer(int neuronCount)
		{
			// Set the neuron count
			this->neuronCount = neuronCount;
		}

		// Public destructor
		~Layer()
		{
			// Deallocate all the matrices
			delete activations;
			delete deltaActivations;
			delete sums;
			delete deltaSums;
			delete weights;
			delete deltaWeights;
			delete bias;
			delete deltaBias;
		}

		// Allocate the activations matrix
		void SetInputLayerActivationMatrix(int neuronCount, int batchSize)
		{
			// Allocate the activations matrix
			activations = new Matrix(neuronCount, batchSize);
		}

		// Set this layer's previous layer
		void SetPreviousLayer(Layer* previousLayer, int trainingSetBatchSize)
		{
			// Create the matrices for this layer
			activations = new Matrix(neuronCount, trainingSetBatchSize);
			deltaActivations = new Matrix(neuronCount, trainingSetBatchSize);

			sums = new Matrix(neuronCount, trainingSetBatchSize);
			deltaSums = new Matrix(neuronCount, trainingSetBatchSize);

			weights = new Matrix(neuronCount, previousLayer->neuronCount);
			deltaWeights = new Matrix(neuronCount, previousLayer->neuronCount);

			bias = new Matrix(neuronCount, 1);
			deltaBias = new Matrix(neuronCount, 1);

			// Set the previous layer
			this->previousLayer = previousLayer;

			// Set the previous layer's next to this layer
			previousLayer->nextLayer = this;
		}

		// Computes the activations for this layer, and causes the nextLayer to FeedForward if it exists
		void FeedForward()
		{
		}

		// Compute and apply deltaWeights and deltaBias for this layer and call the same
		// for the previousLayer if it exists
		void Backprop()
		{
		}
	};
}