#pragma once

namespace GraceAI
{
	class Matrix
	{
		// Elements for this matrix
		float* data;

		// Rows and columns
		int rows, columns;

		// Total elements, rows*cols
		int total;
	public:
		// Public constructor makes a matrix with rows x columns elements
		Matrix(int rows, int columns)
		{
			this->rows = rows;
			this->columns = columns;
			this->total = rows * columns;

			// Aligned malloc for cache line size
			data = (float*)_aligned_malloc(sizeof(float)*total, 64);
		}

		// Public destructor
		~Matrix()
		{
			_aligned_free(data);
		}

		// Getter and setter to access elememnts in functor style
		float& operator()(int row, int col)
		{
			return data[row + col * rows];
		}

		// Standard N^3 Matrix product, *this = a * b
		void MUL(Matrix& a, Matrix& b)
		{
			if (a.columns != b.rows)
				throw "Columns of matrix A must equal rows of matrix B";
		
			if (this == &a || this == &b)
				throw "Destination matrix must be distinct from matrices A and B";

			int sharedDim = a.columns;
			
			for (int r = 0; r < a.rows; r++)
			{
				for (int c = 0; c < b.columns; c++)
				{
					float total = 0.0f;
					for (int d = 0; d < sharedDim; d++)
					{
						total += a(r, d) * b(d, c);
					}

					this->operator()(r, c) = total;
				}
			}
		}

		// Hadamard or element-wise product. Multiplies each element
		// of this matrix with the corresponding elements of a
		void Hadamard(Matrix& a)
		{
			if (this->rows != a.rows || this->columns != a.columns)
				throw "Matrices must of the same dimensions to Hadamard";

			for (int i = 0; i < this->total; i++)
			{
				data[i] *= a.data[i];
			}
		}

		// Element-wise addition. Adds elements from this matrix
		// with the corresponding elements from matrix a
		void ADD(Matrix& a)
		{
			if (this->rows != a.rows || this->columns != a.columns)
				throw "Matrices must of the same dimensions to Add";

			for (int i = 0; i < this->total; i++)
			{
				data[i] += a.data[i];
			}
		}

		// Element-wise subtract. Subtracts elements from matrix a
		// from corresponding elements in this matrix
		void SUB(Matrix& a)
		{
			if (this->rows != a.rows || this->columns != a.columns)
				throw "Matrices must of the same dimensions to Sub";

			for (int i = 0; i < this->total; i++)
			{
				data[i] -= a.data[i];
			}
		}
	};
}