#pragma once

#include <vector>
#include "Layer.h"
#include "TrainingSet.h"

namespace GraceAI
{
	class NeuralNet
	{
		// Vector for all the layers, including the input and output
		std::vector<Layer*> layers;

		// The data for training
		Trainingset* trainingSet;

		int trainingSetBatchSize;

	public:
		// Public constructor
		NeuralNet()
		{
		}

		// Public destructor, deletes and clears the layers
		~NeuralNet()
		{
			// Deallocate layers
			for (Layer* l: layers)
				delete l;

			// Clear the vector
			layers.clear();
		}

		// Allocates the first layer, the input layer. Note: This function must be called before
		// adding any hidden layers.
		void Init(int inputNeuronCount, int trainingSetBatchSize, int outputNeuronCount)
		{
			// Don't allow multiple Init calls
			if (layers.size() != 0)
				throw "Neural Net has an input layer already";

			// Set the training set batch size
			this->trainingSetBatchSize = trainingSetBatchSize;

			// Create the input layer
			Layer* inputLayer = new Layer(inputNeuronCount);

			// Allocate the activation matrix for the input layer
			inputLayer->SetInputLayerActivationMatrix(inputNeuronCount, trainingSetBatchSize);

			// Push input layer onto vector
			layers.push_back(inputLayer);
		}

		// Adds a hidden layer (or the output layer) to the network
		void AddLayer(int neuronCount)
		{
			// Ensure there is an input lay
			if (layers.size() < 1)
				throw "An input layer must be preset prior to adding hidden layers, please call Init";
		
			// Create the hidden layer
			Layer* hiddenLayer = new Layer(neuronCount);

			// Allocate matrices, and set previous/next layers
			hiddenLayer->SetPreviousLayer(layers.back(), trainingSetBatchSize);

			// Push new hidden layer onto back of vector
			layers.push_back(hiddenLayer);
		}

		// Forward propagate values from the input layer to the output
		void ForwardProp()
		{
		}

		// Compute and update weights and biases starting with the final layer
		void Backprop()
		{
		}
	};
}